@@ -42,9 +42,12 @@
 #include <KMenuBar>
 #include <KLineEdit>
 #include <KMenu>
+#include <KToolBar>
+#include <KToolInvocation>
 
 #include <QEvent>
 #include <QWidgetAction>
+#include <QLabel>
 
 #include <TelepathyQt/Account>
 #include <TelepathyQt/ContactCapabilities>
@@ -255,6 +258,20 @@ void ChatWindow::onAudioCallTriggered()
     startAudioCall(currChat->account(), currChat->textChannel()->targetContact());
 }
 
+void ChatWindow::onBlockContactTriggered()
+{
+    ChatWidget *currChat = qobject_cast<ChatWidget*>(m_tabWidget->currentWidget());
+
+    if (!currChat) {
+        return;
+    }
+
+    Tp::ContactPtr contact = currChat->textChannel()->targetContact();
+    if(!contact.isNull()) {
+        contact->block();
+    }
+}
+
 void ChatWindow::onCurrentIndexChanged(int index)
 {
     kDebug() << index;
@@ -270,7 +287,7 @@ void ChatWindow::onCurrentIndexChanged(int index)
     setWindowIcon(currentChatTab->icon());
 
     //call this to update the "Typing.." in window title
-    onUserTypingChanged();
+    onUserTypingChanged(currentChatTab->remoteChatState());
 
     kDebug() << "Current spell dictionary is" << currentChatTab->spellDictionary();
     m_spellDictCombo->setCurrentByDictionary(currentChatTab->spellDictionary());
@@ -296,6 +313,8 @@ void ChatWindow::onCurrentIndexChanged(int index)
         /// TODO re-activate check when invitation to chat has been sorted out
         setInviteToChatEnabled(false);
 
+        toggleBlockButton(currentChatTab->textChannel()->targetContact()->isBlocked());
+
     } else {
         setAudioCallEnabled(false);
         setFileTransferEnabled(false);
@@ -303,8 +322,16 @@ void ChatWindow::onCurrentIndexChanged(int index)
         setShareDesktopEnabled(false);
         /// TODO re-activate check when invitation to chat has been sorted out
         setInviteToChatEnabled(false);
+        setBlockEnabled(false);
 
     }
+
+    // only show enable the action if there are actually previous converstations
+#ifdef TELEPATHY_LOGGER_QT4_FOUND
+    setPreviousConversationsEnabled(currentChatTab->previousConversationAvailable());
+#endif
+
+    setAccountIcon(currentChatTab->accountIcon());
 }
 
 void ChatWindow::onEnableSearchActions(bool enable)
@@ -434,6 +461,18 @@ void ChatWindow::onVideoCallTriggered()
     startVideoCall(currChat->account(), currChat->textChannel()->targetContact());
 }
 
+void ChatWindow::onUnblockContactTriggered()
+{
+    ChatWidget *currChat = qobject_cast<ChatWidget*>(m_tabWidget->currentWidget());
+
+    if(!currChat) {
+        return;
+    }
+
+    Tp::ContactPtr contact = currChat->textChannel()->targetContact();
+    contact->unblock();
+}
+
 void ChatWindow::onShareDesktopTriggered()
 {
     ChatWidget *currChat =  qobject_cast<ChatWidget*>(m_tabWidget->currentWidget());
@@ -446,6 +485,24 @@ void ChatWindow::onShareDesktopTriggered()
     startShareDesktop(currChat->account(), currChat->textChannel()->targetContact());
 }
 
+void ChatWindow::onOpenLogTriggered()
+{
+    int index = m_tabWidget->currentIndex();
+    ChatTab* currentChatTab = qobject_cast<ChatTab*>(m_tabWidget->widget(index));
+    Q_ASSERT(currentChatTab);
+
+    Tp::AccountPtr account = currentChatTab->account();
+    Tp::ContactPtr contact = currentChatTab->textChannel()->targetContact();
+
+    if (!contact.isNull()) {
+        KToolInvocation::kdeinitExec(QLatin1String("ktp-log-viewer"),
+                                     QStringList() << account->uniqueIdentifier() << contact->id());
+    } else {
+        KToolInvocation::kdeinitExec(QLatin1String("ktp-log-viewer"),
+                                     QStringList() << account->uniqueIdentifier() << currentChatTab->textChannel()->targetId());
+    }
+}
+
 
 void ChatWindow::showSettingsDialog()
 {
@@ -469,10 +526,11 @@ void ChatWindow::removeChatTabSignals(ChatTab* chatTab)
 {
     disconnect(chatTab, SIGNAL(titleChanged(QString)), this, SLOT(onTabTextChanged(QString)));
     disconnect(chatTab, SIGNAL(iconChanged(KIcon)), this, SLOT(onTabIconChanged(KIcon)));
-    disconnect(chatTab, SIGNAL(userTypingChanged(bool)), this, SLOT(onTabStateChanged()));
+    disconnect(chatTab, SIGNAL(userTypingChanged(Tp::ChannelChatState)), this, SLOT(onTabStateChanged()));
     disconnect(chatTab, SIGNAL(unreadMessagesChanged()), this, SLOT(onTabStateChanged()));
     disconnect(chatTab, SIGNAL(contactPresenceChanged(Tp::Presence)), this, SLOT(onTabStateChanged()));
     disconnect(chatTab->chatSearchBar(), SIGNAL(enableSearchButtonsSignal(bool)), this, SLOT(onEnableSearchActions(bool)));
+    disconnect(chatTab, SIGNAL(contactBlockStatusChanged(bool)), this, SLOT(toggleBlockButton(bool)));
 }
 
 void ChatWindow::sendNotificationToUser(ChatWindow::NotificationType type, const QString& errorMsg)
@@ -494,11 +552,12 @@ void ChatWindow::setupChatTabSignals(ChatTab *chatTab)
 {
     connect(chatTab, SIGNAL(titleChanged(QString)), this, SLOT(onTabTextChanged(QString)));
     connect(chatTab, SIGNAL(iconChanged(KIcon)), this, SLOT(onTabIconChanged(KIcon)));
-    connect(chatTab, SIGNAL(userTypingChanged(bool)), this, SLOT(onTabStateChanged()));
-    connect(chatTab, SIGNAL(userTypingChanged(bool)), this, SLOT(onUserTypingChanged()));
+    connect(chatTab, SIGNAL(userTypingChanged(Tp::ChannelChatState)), this, SLOT(onTabStateChanged()));
+    connect(chatTab, SIGNAL(userTypingChanged(Tp::ChannelChatState)), this, SLOT(onUserTypingChanged(Tp::ChannelChatState)));
     connect(chatTab, SIGNAL(unreadMessagesChanged()), this, SLOT(onTabStateChanged()));
     connect(chatTab, SIGNAL(contactPresenceChanged(KTp::Presence)), this, SLOT(onTabStateChanged()));
     connect(chatTab->chatSearchBar(), SIGNAL(enableSearchButtonsSignal(bool)), this, SLOT(onEnableSearchActions(bool)));
+    connect(chatTab, SIGNAL(contactBlockStatusChanged(bool)), this, SLOT(toggleBlockButton(bool)));
 }
 
 void ChatWindow::setupCustomActions()
@@ -511,19 +570,29 @@ void ChatWindow::setupCustomActions()
     previousTabAction->setShortcuts(KStandardShortcut::tabPrev());
     connect(previousTabAction, SIGNAL(triggered()), this, SLOT(onPreviousTabActionTriggered()));
 
-    KAction *audioCallAction = new KAction(KIcon(QLatin1String("voicecall")), i18n("&Audio Call"), this);
+    KAction *audioCallAction = new KAction(KIcon(QLatin1String("audio-headset")), i18n("&Audio Call"), this);
+    audioCallAction->setToolTip(i18nc("Toolbar icon tooltip", "Start an audio call with this contact"));
     connect(audioCallAction, SIGNAL(triggered()), this, SLOT(onAudioCallTriggered()));
 
+    KAction *blockContactAction = new KAction(KIcon(QLatin1String("im-ban-kick-user")), i18n("&Block Contact"), this);
+    blockContactAction->setToolTip(i18nc("Toolbar icon tooltip",
+                                         "Blocking means that this contact will not see you online and you will not receive any messages from this contact"));
+    connect(blockContactAction, SIGNAL(triggered()), this, SLOT(onBlockContactTriggered()));
+
     KAction *fileTransferAction = new KAction(KIcon(QLatin1String("mail-attachment")), i18n("&Send File"), this);
+    fileTransferAction->setToolTip(i18nc("Toolbar icon tooltip", "Send a file to this contact"));
     connect(fileTransferAction, SIGNAL(triggered()), this, SLOT(onFileTransferTriggered()));
 
     KAction *inviteToChat = new KAction(KIcon(QLatin1String("user-group-new")), i18n("&Invite to Chat"), this);
+    inviteToChat->setToolTip(i18nc("Toolbar icon tooltip", "Invite any other contacts to join this chat"));
     connect(inviteToChat, SIGNAL(triggered()), this, SLOT(onInviteToChatTriggered()));
 
-    KAction *videoCallAction = new KAction(KIcon(QLatin1String("webcamsend")), i18n("&Video Call"), this);
+    KAction *videoCallAction = new KAction(KIcon(QLatin1String("camera-web")), i18n("&Video Call"), this);
+    videoCallAction->setToolTip(i18nc("Toolbar icon tooltip", "Start a video call with this contact"));
     connect(videoCallAction, SIGNAL(triggered()), this, SLOT(onVideoCallTriggered()));
 
     KAction *shareDesktopAction = new KAction(KIcon(QLatin1String("krfb")), i18n("Share My &Desktop"), this);
+    shareDesktopAction->setToolTip(i18nc("Toolbar icon tooltip", "Start an application that allows this contact to see your desktop"));
     connect(shareDesktopAction, SIGNAL(triggered()), this, SLOT(onShareDesktopTriggered()));
 
     m_spellDictCombo = new Sonnet::DictionaryComboBox();
@@ -536,6 +605,15 @@ void ChatWindow::setupCustomActions()
     spellDictComboAction->setIcon(KIcon(QLatin1String("tools-check-spelling")));
     spellDictComboAction->setIconText(i18n("Choose Spelling Language"));
 
+#ifdef TELEPATHY_LOGGER_QT4_FOUND
+    KAction *openLogAction = new KAction(KIcon(QLatin1String("view-pim-journal")), i18nc("Action to open the log viwer with a specified contact","&Previous conversations"), this);
+    connect(openLogAction, SIGNAL(triggered()), SLOT(onOpenLogTriggered()));
+#endif
+
+    KAction *accountIconAction = new KAction(KIcon(QLatin1String("telepathy-kde")), i18n("Account Icon"), this);
+    m_accountIconLabel = new QLabel(this);
+    accountIconAction->setDefaultWidget(m_accountIconLabel);
+
     // add custom actions to the collection
     actionCollection()->addAction(QLatin1String("next-tab"), nextTabAction);
     actionCollection()->addAction(QLatin1String("previous-tab"), previousTabAction);
@@ -545,6 +623,11 @@ void ChatWindow::setupCustomActions()
     actionCollection()->addAction(QLatin1String("invite-to-chat"), inviteToChat);
     actionCollection()->addAction(QLatin1String("share-desktop"), shareDesktopAction);
     actionCollection()->addAction(QLatin1String("language"), spellDictComboAction);
+    actionCollection()->addAction(QLatin1String("account-icon"), accountIconAction);
+    actionCollection()->addAction(QLatin1String("block-contact"), blockContactAction);
+#ifdef TELEPATHY_LOGGER_QT4_FOUND
+    actionCollection()->addAction(QLatin1String("open-log"), openLogAction);
+#endif
 }
 
 void ChatWindow::setAudioCallEnabled(bool enable)
@@ -557,6 +640,15 @@ void ChatWindow::setAudioCallEnabled(bool enable)
     }
 }
 
+void ChatWindow::setBlockEnabled(bool enable)
+{
+    QAction *action = actionCollection()->action(QLatin1String("block-contact"));
+
+    if (action) {
+        action->setEnabled(enable);
+    }
+}
+
 void ChatWindow::setFileTransferEnabled(bool enable)
 {
     QAction *action = actionCollection()->action(QLatin1String("send-file"));
@@ -593,11 +685,27 @@ void ChatWindow::setShareDesktopEnabled(bool enable)
     }
 }
 
+void ChatWindow::setPreviousConversationsEnabled ( bool enable )
+{
+    QAction *action = actionCollection()->action(QLatin1String("open-log"));
+
+    if (action) {
+        action->setEnabled(enable);
+    }
+}
+
+void ChatWindow::setAccountIcon(const QIcon &icon)
+{
+    m_accountIconLabel->setPixmap(icon.pixmap(toolBar()->iconSize()));
+}
+
 void ChatWindow::startAudioCall(const Tp::AccountPtr& account, const Tp::ContactPtr& contact)
 {
-    Tp::PendingChannelRequest* channelRequest = account->ensureStreamedMediaAudioCall(contact,
-                                                                                      QDateTime::currentDateTime(),
-                                                                                      QLatin1String(PREFERRED_AUDIO_VIDEO_HANDLER));
+    Tp::PendingChannelRequest *channelRequest = account->ensureAudioCall(contact,
+                                                                         QLatin1String("audio"),
+                                                                         QDateTime::currentDateTime(),
+                                                                         QLatin1String(PREFERRED_AUDIO_VIDEO_HANDLER));
+
     connect(channelRequest, SIGNAL(finished(Tp::PendingOperation*)), this, SLOT(onGenericOperationFinished(Tp::PendingOperation*)));
 }
 
@@ -637,10 +745,11 @@ void ChatWindow::startFileTransfer(const Tp::AccountPtr& account, const Tp::Cont
 
 void ChatWindow::startVideoCall(const Tp::AccountPtr& account, const Tp::ContactPtr& contact)
 {
-    Tp::PendingChannelRequest* channelRequest = account->ensureStreamedMediaVideoCall(contact,
-                                                                                      true,
-                                                                                      QDateTime::currentDateTime(),
-                                                                                      QLatin1String(PREFERRED_AUDIO_VIDEO_HANDLER));
+    Tp::PendingChannelRequest* channelRequest = account->ensureAudioVideoCall(contact,
+                                                                              QLatin1String("audio"),
+                                                                              QLatin1String("video"),
+                                                                              QDateTime::currentDateTime(),
+                                                                              QLatin1String(PREFERRED_AUDIO_VIDEO_HANDLER));
 
     connect(channelRequest, SIGNAL(finished(Tp::PendingOperation*)), this, SLOT(onGenericOperationFinished(Tp::PendingOperation*)));
 }
@@ -655,14 +764,16 @@ void ChatWindow::startShareDesktop(const Tp::AccountPtr& account, const Tp::Cont
     connect(channelRequest, SIGNAL(finished(Tp::PendingOperation*)), this, SLOT(onGenericOperationFinished(Tp::PendingOperation*)));
 }
 
-void ChatWindow::onUserTypingChanged()
+void ChatWindow::onUserTypingChanged(Tp::ChannelChatState state)
 {
     ChatWidget *currChat =  qobject_cast<ChatWidget*>(m_tabWidget->currentWidget());
     Q_ASSERT(currChat);
     QString title = currChat->title();
 
-    if (currChat->isUserTyping()) {
+    if (state == Tp::ChannelChatStateComposing) {
         setWindowTitle(i18nc("String prepended in window title, arg is contact's name", "Typing... %1", title));
+    } else if (state == Tp::ChannelChatStatePaused) {
+        setWindowTitle(i18nc("String appended in window title, arg is contact's name", "%1 has entered text", title));
     } else {
         setWindowTitle(title);
     }
@@ -687,5 +798,31 @@ void ChatWindow::setTabSpellDictionary(const QString &dict)
     currentChatTab->setSpellDictionary(dict);
 }
 
+void ChatWindow::toggleBlockButton(bool contactIsBlocked)
+{
+    QAction *action = actionCollection()->action(QLatin1String("block-contact"));
+    if(contactIsBlocked) {
+        //Change the name of the action to "Unblock Contact"
+        //and disconnect it with the block slot and reconnect it with unblock slot
+        disconnect(action, SIGNAL(triggered()), this, SLOT(onBlockContactTriggered()));
+        action->setText(i18n("&Unblock Contact"));
+
+        connect(action, SIGNAL(triggered()), this, SLOT(onUnblockContactTriggered()));
+    } else {
+        //Change the name of the action to "Block Contact"
+        //and disconnect it with the unblock slot and reconnect it with block slot
+        disconnect(action, SIGNAL(triggered()), this, SLOT(onUnblockContactTriggered()));
+        action->setText(i18n("&Block Contact"));
+
+        connect(action, SIGNAL(triggered()), this, SLOT(onBlockContactTriggered()));
+    }
+    //Reset the WindowTitle
+    ChatWidget *currChat =  qobject_cast<ChatWidget*>(m_tabWidget->currentWidget());
+    Q_ASSERT(currChat);
+    setWindowTitle(currChat->title());
+
+    setBlockEnabled(true);
+}
+
 
 #include "chat-window.moc"
