@@ -32,6 +32,7 @@ namespace Sonnet {
 
 class KIcon;
 class ChatTab;
+class QLabel;
 
 class ChatWindow : public KXmlGuiWindow
 {
@@ -78,6 +79,7 @@ private Q_SLOTS:
     void tabBarContextMenu(int  index, const QPoint &  globalPos);
     void closeCurrentTab();
     void onAudioCallTriggered();                                /** start an audio call */
+    void onBlockContactTriggered();                             /** Blocks contact */
     void onCurrentIndexChanged(int index);
     void onEnableSearchActions(bool enable);                    /** enables/disables menu search actions */
     void onFileTransferTriggered();                             /** start a file transfer (to be used only for 1on1 chats!) */
@@ -92,9 +94,12 @@ private Q_SLOTS:
     void onTabTextChanged(const QString &newTitle);
     void onTabIconChanged(const KIcon &newIcon);
     void onVideoCallTriggered();                                /** start a video call */
-    void onUserTypingChanged();
+    void onUserTypingChanged(Tp::ChannelChatState state);
+    void onUnblockContactTriggered();                           /** Unblocks contact when already blocked */
     void onShareDesktopTriggered();                             /** start a desktop share */
+    void onOpenLogTriggered();                                  /** Starts ktp-log-viewer accountId contactId */
     void setTabSpellDictionary(const QString &dict);            /** set the spelling language for the current chat tab*/
+    void toggleBlockButton(bool contactIsBlocked);              /** Toggle block/unblock action according to the flag */
 
 protected Q_SLOTS:
     void showSettingsDialog();
@@ -122,10 +127,15 @@ private:
 
     /** setters for chat actions */
     void setAudioCallEnabled(bool enable);
+    void setBlockEnabled(bool enable);
     void setFileTransferEnabled(bool enable);
     void setInviteToChatEnabled(bool enable);
     void setVideoCallEnabled(bool enable);
     void setShareDesktopEnabled(bool enable);
+    void setPreviousConversationsEnabled(bool enable);
+
+    /** setter for account icon fake action */
+    void setAccountIcon(const QIcon &protocolIcon);
 
     /** starts audio call with given contact
      * @param account account sending the audio call request
@@ -154,6 +164,7 @@ private:
     KTabWidget *m_tabWidget;
 
     Sonnet::DictionaryComboBox *m_spellDictCombo;
+    QLabel *m_accountIconLabel;
 };
 
 #endif // CHATWINDOW_H
