@@ -48,6 +48,17 @@ public:
         SingleUserChat
     };
 
+    enum AppendMode {
+        AppendModeError = 0,
+        AppendMessageWithScroll,
+        AppendNextMessageWithScroll,
+        AppendMessage,
+        AppendNextMessage,
+        AppendMessageNoScroll,
+        AppendNextMessageNoScroll,
+        ReplaceLastMessage
+    };
+
     explicit AdiumThemeView(QWidget *parent = 0);
 
     /** Loads the Theme data*/
@@ -59,10 +70,10 @@ public:
     const QString variantPath() const;
     const QString variantName() const;
     void setVariant(const QString& variant);
-    ChatWindowStyle *chatStyle() const;
+    ChatWindowStyle* chatStyle() const;
 
     /** Set the theme to use. Display will only change once initialise() is called.*/
-    void setChatStyle(ChatWindowStyle* chatStyle);
+    void setChatStyle(ChatWindowStyle *chatStyle);
     void setUseCustomFont(bool);
     QString fontFamily();
     int fontSize();
@@ -88,10 +99,17 @@ private:
     ChatWindowStyle *m_chatStyle;
     QString m_variantPath;
     QString m_variantName;
-    KEmoticons m_emoticons;
     bool m_useCustomFont;
     QString m_fontFamily;
     int m_fontSize;
+
+    QString appendScript(AppendMode mode);
+    AppendMode appendMode(const AdiumThemeMessageInfo &message,
+                          bool consecutive,
+                          bool willAddMoreContentObjects,
+                          bool replaceLastContent);
+    void appendMessage(QString&, AppendMode mode);
+
     QString replaceHeaderKeywords(QString htmlTemplate, const AdiumThemeHeaderInfo&);
     QString replaceContentKeywords(QString& htmlTemplate, const AdiumThemeContentInfo&);
     QString replaceStatusKeywords(QString& htmlTemplate, const AdiumThemeStatusInfo&);
@@ -104,8 +122,6 @@ private:
     bool m_displayHeader;
     KAction *m_openLinkAction;
 
-    void appendNewMessage(QString&);
-    void appendNextMessage(QString&);
 
     bool m_webInspector;
 };
