@@ -46,6 +46,7 @@
 #include <TelepathyQt/Presence>
 
 #include <KTp/presence.h>
+#include "message-processor.h"
 
 class ChatWidgetPrivate
 {
@@ -53,11 +54,11 @@ public:
     ChatWidgetPrivate()
     {
         isGroupChat = false;
-        remoteContactIsTyping = false;
+        remoteContactChatState = Tp::ChannelChatStateInactive;
     }
     /** Stores whether the channel is ready with all contacts upgraded*/
     bool chatviewlInitialised;
-    bool remoteContactIsTyping;
+    Tp::ChannelChatState remoteContactChatState;
     QAction *showFormatToolbarAction;
     bool isGroupChat;
     QString title;
@@ -68,6 +69,7 @@ public:
     Ui::ChatWidget ui;
     ChannelContactModel *contactModel;
     LogManager *logManager;
+    QTimer *pausedStateTimer;
 
     KComponentData telepathyComponentData();
 };
@@ -128,6 +130,7 @@ ChatWidget::ChatWidget(const Tp::TextChannelPtr & channel, const Tp::AccountPtr
 
     AdiumThemeHeaderInfo info;
 
+    info.setGroupChat(d->isGroupChat);
     //normal chat - self and one other person.
     if (d->isGroupChat) {
         info.setChatName(d->channel->targetId());
@@ -151,6 +154,7 @@ ChatWidget::ChatWidget(const Tp::TextChannelPtr & channel, const Tp::AccountPtr
     //set up anything related to 'self'
     info.setOutgoingIconPath(d->channel->groupSelfContact()->avatarData().fileName);
     info.setTimeOpened(QDateTime::currentDateTime());
+    info.setServiceIconImage(KIconLoader::global()->iconPath(d->account->iconName(), KIconLoader::Panel));
     connect(d->ui.chatArea, SIGNAL(loadFinished(bool)), SLOT(chatViewReady()), Qt::QueuedConnection);
     d->ui.chatArea->initialise(info);
 
@@ -170,6 +174,9 @@ ChatWidget::ChatWidget(const Tp::TextChannelPtr & channel, const Tp::AccountPtr
 
     d->ui.sendMessageBox->setSpellCheckingLanguage(KGlobal::locale()->language());
 
+    d->pausedStateTimer = new QTimer(this);
+    d->pausedStateTimer->setSingleShot(true);
+
     //connect signals/slots from format toolbar
     connect(d->ui.formatColor, SIGNAL(released()), SLOT(onFormatColorReleased()));
     connect(d->ui.formatBold, SIGNAL(toggled(bool)), d->ui.sendMessageBox, SLOT(setFontBold(bool)));
@@ -179,7 +186,6 @@ ChatWidget::ChatWidget(const Tp::TextChannelPtr & channel, const Tp::AccountPtr
     // make the sendMessageBox a focus proxy for the chatview
     d->ui.chatArea->setFocusProxy(d->ui.sendMessageBox);
     connect(d->ui.sendMessageBox, SIGNAL(returnKeyPressed()), SLOT(sendMessage()));
-    connect(d->ui.sendButton, SIGNAL(clicked()), SLOT(sendMessage()));
 
     connect(d->ui.searchBar, SIGNAL(findTextSignal(QString,QWebPage::FindFlags)), this, SLOT(findTextInChat(QString,QWebPage::FindFlags)));
     connect(d->ui.searchBar, SIGNAL(findNextSignal(QString,QWebPage::FindFlags)), this, SLOT(findNextTextInChat(QString,QWebPage::FindFlags)));
@@ -188,10 +194,15 @@ ChatWidget::ChatWidget(const Tp::TextChannelPtr & channel, const Tp::AccountPtr
 
     connect(this, SIGNAL(searchTextComplete(bool)), d->ui.searchBar, SLOT(onSearchTextComplete(bool)));
 
+    connect(d->pausedStateTimer, SIGNAL(timeout()), this, SLOT(onChatPausedTimerExpired()));
+
     // initialize LogManager
     if (!d->isGroupChat) {
         d->logManager->setFetchAmount(3);
         d->logManager->setTextChannel(d->account, d->channel);
+        m_previousConversationAvailable = d->logManager->exists();
+    } else {
+        m_previousConversationAvailable = false;
     }
 }
 
@@ -249,12 +260,17 @@ KIcon ChatWidget::icon() const
     return KTp::Presence(Tp::Presence::offline()).icon();
 }
 
+KIcon ChatWidget::accountIcon() const
+{
+    return KIcon(d->account->iconName());
+}
+
 bool ChatWidget::isGroupChat() const
 {
     return d->isGroupChat;
 }
 
-ChatSearchBar* ChatWidget::chatSearchBar() const
+ChatSearchBar *ChatWidget::chatSearchBar() const
 {
     return d->ui.searchBar;
 }
@@ -262,7 +278,6 @@ ChatSearchBar* ChatWidget::chatSearchBar() const
 void ChatWidget::setChatEnabled(bool enable)
 {
     d->ui.sendMessageBox->setEnabled(enable);
-    d->ui.sendButton->setEnabled(enable);
 
     // show a message informing the user
     AdiumThemeStatusInfo statusMessage;
@@ -293,7 +308,7 @@ Tp::TextChannelPtr ChatWidget::textChannel() const
     return d->channel;
 }
 
-void ChatWidget::keyPressEvent(QKeyEvent* e)
+void ChatWidget::keyPressEvent(QKeyEvent *e)
 {
     if (e->matches(QKeySequence::Copy)) {
         d->ui.chatArea->triggerPageAction(QWebPage::Copy);
@@ -330,7 +345,7 @@ QColor ChatWidget::titleColor() const
 
     KColorScheme scheme(QPalette::Active, KColorScheme::Window);
 
-    if (d->remoteContactIsTyping) {
+    if (d->remoteContactChatState == Tp::ChannelChatStateComposing) {
         kDebug() << "remote is typing";
         return scheme.foreground(KColorScheme::PositiveText).color();
     }
@@ -377,7 +392,7 @@ void ChatWidget::setupChannelSignals()
             SLOT(handleMessageSent(Tp::Message,Tp::MessageSendingFlags,QString)));
     connect(d->channel.data(), SIGNAL(chatStateChanged(Tp::ContactPtr,Tp::ChannelChatState)),
             SLOT(onChatStatusChanged(Tp::ContactPtr,Tp::ChannelChatState)));
-    connect(d->channel->connection().data(), SIGNAL(invalidated(Tp::DBusProxy*,QString,QString)),
+    connect(d->channel.data(), SIGNAL(invalidated(Tp::DBusProxy*,QString,QString)),
             this, SLOT(onChannelInvalidated()));
 
     if (d->channel->hasChatStateInterface()) {
@@ -391,6 +406,8 @@ void ChatWidget::setupContactModelSignals()
             SLOT(onContactPresenceChange(Tp::ContactPtr,KTp::Presence)));
     connect(d->contactModel, SIGNAL(contactAliasChanged(Tp::ContactPtr,QString)),
             SLOT(onContactAliasChanged(Tp::ContactPtr,QString)));
+    connect(d->contactModel, SIGNAL(contactBlockStatusChanged(Tp::ContactPtr,bool)),
+       SLOT(onContactBlockStatusChanged(Tp::ContactPtr,bool)));
 }
 
 
@@ -460,12 +477,23 @@ void ChatWidget::handleIncomingMessage(const Tp::ReceivedMessage &message)
                 switch (reportDetails.error()) {
                 case Tp::ChannelTextSendErrorOffline:
                     if (reportDetails.hasEchoedMessage()) {
-                        text = i18n("Delivery of the message \"%1\" "
-                                    "failed because the remote contact is offline",
-                                    reportDetails.echoedMessage().text());
+                        if(message.sender()->isBlocked()) {
+                            text = i18n("Delivery of the message \"%1\" "
+                                        "failed because the remote contact is blocked",
+                                        reportDetails.echoedMessage().text());
+                         } else {
+                            text = i18n("Delivery of the message \"%1\" "
+                                        "failed because the remote contact is offline",
+                                        reportDetails.echoedMessage().text());
+                         }
                     } else {
-                        text = i18n("Delivery of a message failed "
-                                    "because the remote contact is offline");
+                        if(message.sender()->isBlocked()) {
+                            text = i18n("Delivery of a message failed "
+                                        "because the remote contact is blocked");
+                        } else {
+                            text = i18n("Delivery of a message failed "
+                                        "because the remote contact is offline");
+                        }
                     }
                     break;
                 case Tp::ChannelTextSendErrorInvalidContact:
@@ -538,7 +566,7 @@ void ChatWidget::handleIncomingMessage(const Tp::ReceivedMessage &message)
         } else {
             AdiumThemeContentInfo messageInfo(AdiumThemeMessageInfo::RemoteToLocal);
 
-            messageInfo.setMessage(message.text());
+            messageInfo.setMessage(MessageProcessor::instance()->processIncomingMessage(message).finalizedMessage());
 
             QDateTime time = message.sent();
             if (!time.isValid()) {
@@ -582,15 +610,20 @@ void ChatWidget::notifyAboutIncomingMessage(const Tp::ReceivedMessage & message)
         return;
     }
     // don't notify of messages sent by self from another computer
-    if (message.sender() == d->channel->connection()->selfContact()) {
+    if (message.sender() == d->channel->groupSelfContact()) {
         return;
     }
+    
+    if (message.isDeliveryReport()) {
+        return;
+    }
+    
     // kde_telepathy_contact_highlight (contains your name)
     // kde_telepathy_info_event
 
     //if the message text contains sender name, it's a "highlighted message"
     //TODO DrDanz suggested this could be a configurable list of words that make it highlighted.(seems like a good idea to me)
-    if(message.text().contains(d->channel->connection()->selfContact()->alias())) {
+    if(message.text().contains(d->channel->groupSelfContact()->alias())) {
         notificationType = QLatin1String("kde_telepathy_contact_highlight");
     } else if(message.messageType() == Tp::ChannelTextMessageTypeNotice) {
         notificationType = QLatin1String("kde_telepathy_info_event");
@@ -599,7 +632,10 @@ void ChatWidget::notifyAboutIncomingMessage(const Tp::ReceivedMessage & message)
     }
 
 
-    KNotification *notification = new KNotification(notificationType, this);
+    KNotification *notification = new KNotification(notificationType, this,
+                                                    KNotification::RaiseWidgetOnActivation
+                                                    | KNotification::CloseWhenWidgetActivated
+                                                    | KNotification::Persistent);
     notification->setComponentData(d->telepathyComponentData());
     notification->setTitle(i18n("%1 has sent you a message", message.sender()->alias()));
 
@@ -608,13 +644,16 @@ void ChatWidget::notifyAboutIncomingMessage(const Tp::ReceivedMessage & message)
         notification->setPixmap(notificationPixmap);
     }
 
-    if (message.text().length() > 170) {
+    // Remove empty lines from message
+    QString notifyText = message.text().simplified();
+    if (notifyText.length() > 170) {
         //search for the closest space in text
-        QString truncatedMsg = message.text().left(message.text().indexOf(QLatin1Char(' '), 150)).append(QLatin1String("..."));
+        QString truncatedMsg = notifyText.left(notifyText.indexOf(QLatin1Char(' '), 150)).append(QLatin1String("..."));
         notification->setText(truncatedMsg);
     } else {
-        notification->setText(message.text());
+        notification->setText(notifyText);
     }
+
     //allows per contact notifications
     notification->addContext(QLatin1String("contact"), message.sender()->id());
     //TODO notification->addContext("group",... Requires KDE Telepathy Contact to work out which group they are in.
@@ -627,7 +666,7 @@ void ChatWidget::notifyAboutIncomingMessage(const Tp::ReceivedMessage & message)
 
 void ChatWidget::handleMessageSent(const Tp::Message &message, Tp::MessageSendingFlags, const QString&) /*Not sure what these other args are for*/
 {
-    Tp::ContactPtr sender = d->channel->connection()->selfContact();
+    Tp::ContactPtr sender = d->channel->groupSelfContact();
 
     if (message.messageType() == Tp::ChannelTextMessageTypeAction) {
         AdiumThemeStatusInfo statusMessage;
@@ -637,7 +676,8 @@ void ChatWidget::handleMessageSent(const Tp::Message &message, Tp::MessageSendin
     }
     else {
         AdiumThemeContentInfo messageInfo(AdiumThemeMessageInfo::LocalToRemote);
-        messageInfo.setMessage(message.text());
+        messageInfo.setMessage(MessageProcessor::instance()->processOutgoingMessage(message).finalizedMessage());
+
         messageInfo.setTime(message.sent());
 
         messageInfo.setSenderDisplayName(sender->alias());
@@ -684,53 +724,44 @@ void ChatWidget::sendMessage()
 void ChatWidget::onChatStatusChanged(const Tp::ContactPtr & contact, Tp::ChannelChatState state)
 {
     //don't show our own status changes.
-    if (contact == d->channel->connection()->selfContact()) {
+    if (contact == d->channel->groupSelfContact()) {
         return;
     }
 
-    bool contactIsTyping = false;
-
-    switch (state) {
-    case Tp::ChannelChatStateGone:
-      {
+    if (state == Tp::ChannelChatStateGone) {
         AdiumThemeStatusInfo statusMessage;
         statusMessage.setMessage(i18n("%1 has left the chat", contact->alias()));
         statusMessage.setService(d->channel->connection()->protocolName());
         statusMessage.setStatus(QLatin1String("away"));
         statusMessage.setTime(QDateTime::currentDateTime());
         d->ui.chatArea->addStatusMessage(statusMessage);
-        break;
-      }
-    case Tp::ChannelChatStateInactive:
-        //FIXME send a 'chat timed out' message to chatview
-        break;
-    case Tp::ChannelChatStateActive:
-    case Tp::ChannelChatStatePaused:
-        break;
-    case Tp::ChannelChatStateComposing:
-        contactIsTyping = true;
-        break;
-    default:
-        kWarning() << "Unknown channel chat case" << state;
     }
 
-    if (!contactIsTyping) {
+    if (d->isGroupChat) {
         //In a multiperson chat just because this user is no longer typing it doesn't mean that no-one is.
         //loop through each contact, check no-one is in composing mode.
+
+        Tp::ChannelChatState tempState = Tp::ChannelChatStateInactive;
+
         Q_FOREACH (const Tp::ContactPtr & contact, d->channel->groupContacts()) {
-            if (contact == d->channel->connection()->selfContact()) {
+            if (contact == d->channel->groupSelfContact()) {
                 continue;
             }
 
-            if (d->channel->chatState(contact) == Tp::ChannelChatStateComposing) {
-                contactIsTyping = true;
+            tempState = d->channel->chatState(contact);
+
+            if (tempState == Tp::ChannelChatStateComposing) {
+                state = tempState;
+                break;
+            } else if (tempState == Tp::ChannelChatStatePaused && state != Tp::ChannelChatStateComposing) {
+                state = tempState;
             }
         }
     }
 
-    if (contactIsTyping != d->remoteContactIsTyping) {
-        d->remoteContactIsTyping = contactIsTyping;
-        Q_EMIT userTypingChanged(contactIsTyping);
+    if (state != d->remoteContactChatState) {
+        d->remoteContactChatState = state;
+        Q_EMIT userTypingChanged(state);
     }
 }
 
@@ -805,6 +836,24 @@ void ChatWidget::onContactAliasChanged(const Tp::ContactPtr & contact, const QSt
     }
 }
 
+void ChatWidget::onContactBlockStatusChanged(const Tp::ContactPtr &contact, bool blocked)
+{
+    QString message;
+    if(blocked) {
+        message = i18n("%1 is now blocked.", contact->alias());
+    } else {
+        message = i18n("%1 is now unblocked.", contact->alias());
+    }
+
+    AdiumThemeStatusInfo statusMessage;
+    statusMessage.setMessage(message);
+    statusMessage.setService(d->channel->connection()->protocolName());
+    statusMessage.setTime(QDateTime::currentDateTime());
+    d->ui.chatArea->addStatusMessage(statusMessage);
+
+    Q_EMIT contactBlockStatusChanged(blocked);
+}
+
 void ChatWidget::onChannelInvalidated()
 {
     setChatEnabled(false);
@@ -813,14 +862,15 @@ void ChatWidget::onChannelInvalidated()
 void ChatWidget::onInputBoxChanged()
 {
     //if the box is empty
-    bool currentlyTyping = !d->ui.sendMessageBox->toPlainText().isEmpty();
+    bool textBoxEmpty = !d->ui.sendMessageBox->toPlainText().isEmpty();
 
     //FIXME buffer what we've sent to telepathy, make this more efficient.
-    //FIXME check spec (with olly) as to whether we have to handle idle state etc.
-    if(currentlyTyping) {
+    if(textBoxEmpty) {
         d->channel->requestChatState(Tp::ChannelChatStateComposing);
+        d->pausedStateTimer->start(5000);
     } else {
         d->channel->requestChatState(Tp::ChannelChatStateActive);
+        d->pausedStateTimer->stop();
     }
 }
 
@@ -855,11 +905,6 @@ void ChatWidget::onFormatColorReleased()
     d->ui.sendMessageBox->setTextColor(color);
 }
 
-bool ChatWidget::isUserTyping() const
-{
-    return d->remoteContactIsTyping;
-}
-
 void ChatWidget::setSpellDictionary(const QString &dict)
 {
     d->ui.sendMessageBox->setSpellCheckingLanguage(dict);
@@ -870,5 +915,19 @@ QString ChatWidget::spellDictionary() const
     return d->ui.sendMessageBox->spellCheckingLanguage();
 }
 
+Tp::ChannelChatState ChatWidget::remoteChatState()
+{
+    return d->remoteContactChatState;
+}
+
+bool ChatWidget::previousConversationAvailable()
+{
+    return m_previousConversationAvailable;
+}
+
+void ChatWidget::onChatPausedTimerExpired()
+{
+        d->channel->requestChatState(Tp::ChannelChatStatePaused);
+}
 
 #include "chat-widget.moc"
