@@ -71,7 +71,16 @@ LogManager::~LogManager()
 bool LogManager::exists() const
 {
 #ifdef TELEPATHY_LOGGER_QT4_FOUND
-    return m_logManager->exists(m_account, m_contactEntity, Tpl::EventTypeMaskText);
+    if (!m_account.isNull() && !m_textChannel.isNull() && m_textChannel->targetHandleType() == Tp::HandleTypeContact) {
+        Tpl::EntityPtr contactEntity = Tpl::Entity::create(m_textChannel->targetContact()->id().toLatin1().data(),
+                                                           Tpl::EntityTypeContact,
+                                                           NULL,
+                                                           NULL);
+
+        return m_logManager->exists(m_account, contactEntity, Tpl::EventTypeMaskText);
+    } else {
+        return false;
+    }
 #else
     return false;
 #endif
@@ -102,16 +111,14 @@ void LogManager::fetchLast()
         connect(dates, SIGNAL(finished(Tpl::PendingOperation*)), SLOT(onDatesFinished(Tpl::PendingOperation*)));
         return;
     }
+#endif
     //in all other cases finish immediately.
-#else
     QList<AdiumThemeContentInfo> messages;
     Q_EMIT fetched(messages);
-#endif
-
 }
 
 #ifdef TELEPATHY_LOGGER_QT4_FOUND
-void LogManager::onDatesFinished(Tpl::PendingOperation* po)
+void LogManager::onDatesFinished(Tpl::PendingOperation *po)
 {
     Tpl::PendingDates *pd = (Tpl::PendingDates*) po;
 
@@ -127,7 +134,7 @@ void LogManager::onDatesFinished(Tpl::PendingOperation* po)
 
         kDebug() << pd->account()->uniqueIdentifier() << pd->entity()->identifier() << dates;
 
-        Tpl::PendingEvents* events = m_logManager->queryEvents( pd->account(), pd->entity(), Tpl::EventTypeMaskAny, date);
+        Tpl::PendingEvents *events = m_logManager->queryEvents( pd->account(), pd->entity(), Tpl::EventTypeMaskAny, date);
         connect(events, SIGNAL(finished(Tpl::PendingOperation*)), SLOT(onEventsFinished(Tpl::PendingOperation*)));
     } else {
         QList<AdiumThemeContentInfo> messages;
@@ -135,7 +142,7 @@ void LogManager::onDatesFinished(Tpl::PendingOperation* po)
     }
 }
 
-void LogManager::onEventsFinished(Tpl::PendingOperation* po)
+void LogManager::onEventsFinished(Tpl::PendingOperation *po)
 {
     Tpl::PendingEvents *pe = (Tpl::PendingEvents*) po;
 
@@ -184,7 +191,10 @@ void LogManager::onEventsFinished(Tpl::PendingOperation* po)
         iconPath = contact->avatarData().fileName;
 
         AdiumThemeContentInfo message(type);
-        message.setMessage(event->message());
+        // "\" characters are replaced with "\\" to match with normal messages
+        // FIXME we should remove this when the history messages are filtered
+        //       through the message processor
+        message.setMessage(event->message().replace(QLatin1Char('\\'), QLatin1String("\\\\")));
         message.setService(m_account->serviceName());
         message.setSenderDisplayName(event->sender()->alias());
         message.setSenderScreenName(event->sender()->alias());
